@@ -1,7 +1,7 @@
 /* mpfr_hypot -- Euclidean distance
 
-Copyright 2001-2015 Free Software Foundation, Inc.
-Contributed by the AriC and Caramel projects, INRIA.
+Copyright 2001-2017 Free Software Foundation, Inc.
+Contributed by the AriC and Caramba projects, INRIA.
 
 This file is part of the GNU MPFR Library.
 
@@ -98,7 +98,13 @@ mpfr_hypot (mpfr_ptr z, mpfr_srcptr x, mpfr_srcptr y, mpfr_rnd_t rnd_mode)
           /* If z > abs(x), then it was already rounded up; otherwise
              z = abs(x), and we need to add one ulp due to y. */
           if (mpfr_abs (z, x, rnd_mode) == 0)
-            mpfr_nexttoinf (z);
+            {
+              mpfr_nexttoinf (z);
+              /* since mpfr_nexttoinf does not set the overflow flag,
+                 we have to check manually for overflow */
+              if (MPFR_UNLIKELY (MPFR_IS_INF (z)))
+                MPFR_SET_OVERFLOW ();
+            }
           MPFR_RET (1);
         }
       else /* MPFR_RNDZ, MPFR_RNDD, MPFR_RNDN */
@@ -111,7 +117,7 @@ mpfr_hypot (mpfr_ptr z, mpfr_srcptr x, mpfr_srcptr y, mpfr_rnd_t rnd_mode)
           else
             {
               MPFR_SET_EXP (z, Ex);
-              MPFR_SET_SIGN (z, 1);
+              MPFR_SET_SIGN (z, MPFR_SIGN_POS);
               MPFR_RNDRAW_GEN (inexact, z, MPFR_MANT (x), N, rnd_mode, 1,
                                goto addoneulp,
                                if (MPFR_UNLIKELY (++ MPFR_EXP (z) >
@@ -139,11 +145,29 @@ mpfr_hypot (mpfr_ptr z, mpfr_srcptr x, mpfr_srcptr y, mpfr_rnd_t rnd_mode)
 
   MPFR_SAVE_EXPO_MARK (expo);
 
-  /* Scale x and y to avoid overflow/underflow in x^2 and overflow in y^2
-     (as |x| >= |y|). The scaling of y can underflow only when the target
-     precision is huge, otherwise the case would already have been handled
-     by the diff_exp > threshold code. */
-  sh = mpfr_get_emax () / 2 - Ex - 1;
+  /* Scale x and y to avoid any overflow and underflow in x^2
+     (as |x| >= |y|), and to limit underflow cases for y or y^2.
+     We have: x = Mx * 2^Ex with 1/2 <= |Mx| < 1, and we choose:
+     sh = floor((Emax - 1) / 2) - Ex.
+     Thus (x * 2^sh)^2 = Mx^2 * 2^(2*floor((Emax-1)/2)), which has an
+     exponent of at most Emax - 1. Therefore (x * 2^sh)^2 + (y * 2^sh)^2
+     will have an exponent of at most Emax, even after rounding as we
+     round toward zero. Using a larger sh wouldn't guarantee an absence
+     of overflow. Note that the scaling of y can underflow only when the
+     target precision is huge, otherwise the case would already have been
+     handled by the diff_exp > threshold code.
+     FIXME: Friedland in "Algorithm 312: Absolute Value and Square Root of a
+     Complex Number" (Communications of the ACM, 1967) avoids overflow by
+     computing |x|*sqrt(1+(y/x)^2) if |x| >= |y|, and |y|*sqrt(1+(x/y)^2)
+     otherwise.
+     [VL] This trick (which is a scaling by a non-power of 2, thus doesn't
+     really bring new behavior w.r.t. overflow/underflow exceptions) may be
+     useful for hardware floating-point formats because a whole power-of-2
+     scaling code is likely to take more time than the additional division,
+     but in the context of multiple-precision, I doubt that it is a good
+     idea. Ideally scaling by a power of 2 could be done in a constant time,
+     e.g. with MPFR_ALIAS; but one needs to be very careful... */
+  sh = (mpfr_get_emax () - 1) / 2 - Ex;
 
   MPFR_ZIV_INIT (loop, Nt);
   for (;;)
@@ -170,7 +194,7 @@ mpfr_hypot (mpfr_ptr z, mpfr_srcptr x, mpfr_srcptr y, mpfr_rnd_t rnd_mode)
   MPFR_ZIV_FREE (loop);
 
   MPFR_BLOCK (flags, inexact = mpfr_div_2si (z, t, sh, rnd_mode));
-  MPFR_ASSERTD (exact == 0 || inexact != 0);
+  MPFR_ASSERTD (exact == 0 || inexact != 0 || rnd_mode == MPFR_RNDF);
 
   mpfr_clear (t);
   mpfr_clear (ti);
@@ -187,7 +211,7 @@ mpfr_hypot (mpfr_ptr z, mpfr_srcptr x, mpfr_srcptr y, mpfr_rnd_t rnd_mode)
   MPFR_SAVE_EXPO_FREE (expo);
 
   if (MPFR_OVERFLOW (flags))
-    mpfr_set_overflow ();
+    MPFR_SET_OVERFLOW ();
   /* hypot(x,y) >= |x|, thus underflow is not possible. */
 
   return mpfr_check_range (z, inexact, rnd_mode);
