@@ -20,8 +20,6 @@ along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
 http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
 
-#include <stdio.h>
-#include <stdlib.h>
 #include <float.h>
 
 #include "mpfr-test.h"
@@ -97,6 +95,8 @@ special (void)
   mpfr_nextabove (x);
   mpfr_div_ui (y, x, 2, MPFR_RNDN); /* exactly in the middle */
   MPFR_ASSERTN(mpfr_cmp_ui (y, 2) == 0);
+  (mpfr_div_ui) (y, x, 2, MPFR_RNDN); /* exactly in the middle */
+  MPFR_ASSERTN(mpfr_cmp_ui (y, 2) == 0);
 
   mpfr_set_prec (x, 3 * mp_bits_per_limb);
   mpfr_set_prec (y, 2);
@@ -104,6 +104,8 @@ special (void)
   mpfr_nextabove (x);
   mpfr_div_ui (y, x, 2, MPFR_RNDN);
   MPFR_ASSERTN(mpfr_cmp_ui (y, 1) == 0);
+  (mpfr_div_ui) (y, x, 2, MPFR_RNDN);
+  MPFR_ASSERTN(mpfr_cmp_ui (y, 1) == 0);
 
   mpfr_set_prec (x, 3 * mp_bits_per_limb);
   mpfr_set_prec (y, 2);
@@ -111,6 +113,8 @@ special (void)
   mpfr_nextbelow (x);
   mpfr_div_ui (y, x, 2, MPFR_RNDD);
   MPFR_ASSERTN(mpfr_cmp_si (y, -3) == 0);
+  (mpfr_div_ui) (y, x, 2, MPFR_RNDD);
+  MPFR_ASSERTN(mpfr_cmp_si (y, -3) == 0);
 
   for (xprec = 53; xprec <= 128; xprec++)
     {
@@ -123,8 +127,8 @@ special (void)
           if (mpfr_cmp(x,y))
             {
               printf ("division by 1.0 fails for xprec=%u, yprec=%u\n", xprec, yprec);
-              printf ("expected "); mpfr_print_binary (x); puts ("");
-              printf ("got      "); mpfr_print_binary (y); puts ("");
+              printf ("expected "); mpfr_dump (x);
+              printf ("got      "); mpfr_dump (y);
               exit (1);
             }
         }
@@ -135,6 +139,8 @@ special (void)
   mpfr_set_si (y, -1, MPFR_RNDN);
   mpfr_div_ui (y, x, 4, MPFR_RNDN);
   MPFR_ASSERTN(MPFR_IS_ZERO(y) && MPFR_IS_POS(y));
+  (mpfr_div_ui) (y, x, 4, MPFR_RNDN);
+  MPFR_ASSERTN(MPFR_IS_ZERO(y) && MPFR_IS_POS(y));
 
   mpfr_clear (x);
   mpfr_clear (y);
@@ -172,8 +178,8 @@ check_inexact (void)
               if (mpfr_mul_ui (z, y, u, (mpfr_rnd_t) rnd))
                 {
                   printf ("z <- y * u should be exact for u=%lu\n", u);
-                  printf ("y="); mpfr_print_binary (y); puts ("");
-                  printf ("z="); mpfr_print_binary (z); puts ("");
+                  printf ("y="); mpfr_dump (y);
+                  printf ("z="); mpfr_dump (z);
                   exit (1);
                 }
               cmp = mpfr_cmp (z, x);
@@ -183,8 +189,8 @@ check_inexact (void)
                 {
                   printf ("Wrong inexact flag for u=%lu, rnd=%s\n", u,
                           mpfr_print_rnd_mode ((mpfr_rnd_t) rnd));
-                  printf ("x="); mpfr_print_binary (x); puts ("");
-                  printf ("y="); mpfr_print_binary (y); puts ("");
+                  printf ("x="); mpfr_dump (x);
+                  printf ("y="); mpfr_dump (y);
                   exit (1);
                 }
             }
@@ -196,6 +202,42 @@ check_inexact (void)
   mpfr_clear (z);
 }
 
+#if GMP_NUMB_BITS == 64
+/* With r11140, on a 64-bit machine with GMP_CHECK_RANDOMIZE=1484406128:
+   Consistency error for i = 2577
+*/
+static void
+test_20170105 (void)
+{
+  mpfr_t x,z, t;
+
+  if (sizeof (unsigned long) * CHAR_BIT == 64)
+    {
+      mpfr_init2 (x, 138);
+      mpfr_init2 (z, 128);
+      mpfr_init2 (t, 128);
+      mpfr_set_str_binary (x, "100110111001001000101111010010011101111110111111110001110100000001110111010100111010100011101010110000010100000011100100110101101011000000");
+      /* up to exponents, x/y is exactly
+         367625553447399614694201910705139062483, which has 129 bits,
+         thus we are in the round-to-nearest-even case, and since the
+         penultimate bit of x/y is 1, we should round upwards */
+      mpfr_set_str_binary (t, "10001010010010010000110110010110111111111100011011101010000000000110101000010001011110011011010000111010000000001100101101101010E-53");
+      mpfr_div_ui (z, x, 36UL << 58, MPFR_RNDN);
+      MPFR_ASSERTN(mpfr_equal_p (z, t));
+
+      mpfr_set_prec (x, 189);
+      mpfr_set_prec (z, 185);
+      mpfr_set_prec (t, 185);
+      mpfr_set_str_binary (x, "100001010000111100011110111010000011110000000110100010001010101011110001110000110111101000100100001101010011000111110100011111110110011011101000000000001010010010111011001100111111111101001");
+      mpfr_set_str_binary (t, "10011000000100010100011111100100110101101110001011100101010101011010011010010110010000100111001010000101111011111111001011011010101111101100000000000000101111000100001110101001001001000E-60");
+      mpfr_div_ui (z, x, 7UL << 61, MPFR_RNDN);
+      MPFR_ASSERTN(mpfr_equal_p (z, t));
+
+      mpfr_clears (x, z, t, (mpfr_ptr) 0);
+    }
+}
+#endif
+
 #define TEST_FUNCTION mpfr_div_ui
 #define ULONG_ARG2
 #define RAND_FUNCTION(x) mpfr_random2(x, MPFR_LIMB_SIZE (x), 1, RANDS)
@@ -218,6 +260,9 @@ main (int argc, char **argv)
   check("1.0", 3, MPFR_RNDD, "3.3333333333333331483e-1");
   check("1.0", 2116118, MPFR_RNDN, "4.7256343927890600483e-7");
   check("1.098612288668109782", 5, MPFR_RNDN, "0.21972245773362195087");
+#if GMP_NUMB_BITS == 64
+  test_20170105 ();
+#endif
 
   mpfr_init2 (x, 53);
   mpfr_set_ui (x, 3, MPFR_RNDD);
@@ -230,7 +275,7 @@ main (int argc, char **argv)
     }
   mpfr_clear (x);
 
-  test_generic (2, 200, 100);
+  test_generic (MPFR_PREC_MIN, 200, 100);
 
   tests_end_mpfr ();
   return 0;
