@@ -1,7 +1,7 @@
 /* Test file for mpfr_log2.
 
-Copyright 2001-2002, 2004, 2006-2015 Free Software Foundation, Inc.
-Contributed by the AriC and Caramel projects, INRIA.
+Copyright 2001-2002, 2004, 2006-2017 Free Software Foundation, Inc.
+Contributed by the AriC and Caramba projects, INRIA.
 
 This file is part of the GNU MPFR Library.
 
@@ -20,9 +20,6 @@ along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
 http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
 
-#include <stdio.h>
-#include <stdlib.h>
-
 #include "mpfr-test.h"
 
 #define TEST_FUNCTION mpfr_log2
@@ -75,7 +72,7 @@ main (int argc, char *argv[])
 
   special ();
 
-  test_generic (2, 100, 30);
+  test_generic (MPFR_PREC_MIN, 100, 30);
 
   data_check ("data/log2", mpfr_log2, "mpfr_log2");
 
