@@ -1,4 +1,4 @@
-/* Test file for mpfr_set_si and mpfr_set_ui.
+/* Test file for mpfr_set_si, mpfr_set_ui, mpfr_get_si and mpfr_get_ui.
 
 Copyright 1999, 2001-2017 Free Software Foundation, Inc.
 Contributed by the AriC and Caramba projects, INRIA.
@@ -20,13 +20,10 @@ along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
 http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <limits.h>
-
 #include "mpfr-test.h"
 
-#define ERROR(str) do { printf ("Error for " str "\n"); exit (1); } while (0)
+#define PRINT_ERROR(str) \
+  do { printf ("Error for %s\n", str); exit (1); } while (0)
 
 static void
 test_2exp (void)
@@ -37,58 +34,58 @@ test_2exp (void)
   mpfr_init2 (x, 32);
 
   mpfr_set_ui_2exp (x, 1, 0, MPFR_RNDN);
-  if (mpfr_cmp_ui(x, 1))
-    ERROR("(1U,0)");
+  if (mpfr_cmp_ui (x, 1) != 0)
+    PRINT_ERROR ("(1U,0)");
 
   mpfr_set_ui_2exp (x, 1024, -10, MPFR_RNDN);
-  if (mpfr_cmp_ui(x, 1))
-    ERROR("(1024U,-10)");
+  if (mpfr_cmp_ui(x, 1) != 0)
+    PRINT_ERROR ("(1024U,-10)");
 
   mpfr_set_ui_2exp (x, 1024, 10, MPFR_RNDN);
-  if (mpfr_cmp_ui(x, 1024*1024))
-    ERROR("(1024U,+10)");
+  if (mpfr_cmp_ui (x, 1024 * 1024) != 0)
+    PRINT_ERROR ("(1024U,+10)");
 
   mpfr_set_si_2exp (x, -1024L * 1024L, -10, MPFR_RNDN);
-  if (mpfr_cmp_si(x, -1024))
-    ERROR("(1M,-10)");
+  if (mpfr_cmp_si (x, -1024) != 0)
+    PRINT_ERROR ("(1M,-10)");
 
   mpfr_set_ui_2exp (x, 0x92345678, 16, MPFR_RNDN);
-  if (mpfr_cmp_str (x, "92345678@4", 16, MPFR_RNDN))
-    ERROR("(x92345678U,+16)");
+  if (mpfr_cmp_str (x, "92345678@4", 16, MPFR_RNDN) != 0)
+    PRINT_ERROR ("(x92345678U,+16)");
 
   mpfr_set_si_2exp (x, -0x1ABCDEF0, -256, MPFR_RNDN);
-  if (mpfr_cmp_str (x, "-1ABCDEF0@-64", 16, MPFR_RNDN))
-    ERROR("(-x1ABCDEF0,-256)");
+  if (mpfr_cmp_str (x, "-1ABCDEF0@-64", 16, MPFR_RNDN) != 0)
+    PRINT_ERROR ("(-x1ABCDEF0,-256)");
 
   mpfr_set_prec (x, 2);
   res = mpfr_set_si_2exp (x, 7, 10, MPFR_RNDU);
-  if (mpfr_cmp_ui (x, 1<<13) || res <= 0)
-    ERROR ("Prec 2 + si_2exp");
+  if (mpfr_cmp_ui (x, 1<<13) != 0 || res <= 0)
+    PRINT_ERROR ("Prec 2 + si_2exp");
 
   res = mpfr_set_ui_2exp (x, 7, 10, MPFR_RNDU);
-  if (mpfr_cmp_ui (x, 1<<13) || res <= 0)
-    ERROR ("Prec 2 + ui_2exp");
+  if (mpfr_cmp_ui (x, 1<<13) != 0 || res <= 0)
+    PRINT_ERROR ("Prec 2 + ui_2exp");
 
   mpfr_clear_flags ();
   mpfr_set_ui_2exp (x, 17, MPFR_EMAX_MAX, MPFR_RNDN);
   if (!mpfr_inf_p (x) || MPFR_IS_NEG (x))
-    ERROR ("mpfr_set_ui_2exp and overflow (bad result)");
+    PRINT_ERROR ("mpfr_set_ui_2exp and overflow (bad result)");
   if (!mpfr_overflow_p ())
-    ERROR ("mpfr_set_ui_2exp and overflow (overflow flag not set)");
+    PRINT_ERROR ("mpfr_set_ui_2exp and overflow (overflow flag not set)");
 
   mpfr_clear_flags ();
   mpfr_set_si_2exp (x, 17, MPFR_EMAX_MAX, MPFR_RNDN);
   if (!mpfr_inf_p (x) || MPFR_IS_NEG (x))
-    ERROR ("mpfr_set_si_2exp (pos) and overflow (bad result)");
+    PRINT_ERROR ("mpfr_set_si_2exp (pos) and overflow (bad result)");
   if (!mpfr_overflow_p ())
-    ERROR ("mpfr_set_si_2exp (pos) and overflow (overflow flag not set)");
+    PRINT_ERROR ("mpfr_set_si_2exp (pos) and overflow (overflow flag not set)");
 
   mpfr_clear_flags ();
   mpfr_set_si_2exp (x, -17, MPFR_EMAX_MAX, MPFR_RNDN);
   if (!mpfr_inf_p (x) || MPFR_IS_POS (x))
-    ERROR ("mpfr_set_si_2exp (neg) and overflow (bad result)");
+    PRINT_ERROR ("mpfr_set_si_2exp (neg) and overflow (bad result)");
   if (!mpfr_overflow_p ())
-    ERROR ("mpfr_set_si_2exp (neg) and overflow (overflow flag not set)");
+    PRINT_ERROR ("mpfr_set_si_2exp (neg) and overflow (overflow flag not set)");
 
   mpfr_clear (x);
 }
@@ -100,6 +97,9 @@ test_macros (void)
   mpfr_ptr p;
   int r;
 
+  /* Note: the ++'s below allow one to check that the corresponding
+     arguments are evaluated only once by the macros. */
+
   mpfr_inits (x[0], x[1], x[2], (mpfr_ptr) 0);
   p = x[0];
   r = 0;
@@ -143,6 +143,190 @@ test_macros_keyword (void)
   mpfr_clear (x);
 }
 
+static void
+test_get_ui_smallneg (void)
+{
+  mpfr_t x;
+  int i;
+
+  mpfr_init2 (x, 64);
+
+  for (i = 1; i <= 4; i++)
+    {
+      int r;
+
+      mpfr_set_si_2exp (x, -i, -2, MPFR_RNDN);
+      RND_LOOP (r)
+        {
+          long s;
+          unsigned long u;
+
+          mpfr_clear_erangeflag ();
+          s = mpfr_get_si (x, r != MPFR_RNDF ? (mpfr_rnd_t) r : MPFR_RNDA);
+          if (mpfr_erangeflag_p ())
+            {
+              printf ("ERROR for get_si + ERANGE + small negative op"
+                      " for rnd = %s and x = -%d/4\n",
+                      mpfr_print_rnd_mode ((mpfr_rnd_t) r), i);
+              exit (1);
+            }
+          u = mpfr_get_ui (x, (mpfr_rnd_t) r);
+          if (u != 0)
+            {
+              printf ("ERROR for get_ui + ERANGE + small negative op"
+                      " for rnd = %s and x = -%d/4\n",
+                      mpfr_print_rnd_mode ((mpfr_rnd_t) r), i);
+              printf ("Expected 0, got %lu\n", u);
+              exit (1);
+            }
+          if ((s == 0) ^ !mpfr_erangeflag_p ())
+            {
+              const char *Not = s == 0 ? "" : " not";
+
+              printf ("ERROR for get_ui + ERANGE + small negative op"
+                      " for rnd = %s and x = -%d/4\n",
+                      mpfr_print_rnd_mode ((mpfr_rnd_t) r), i);
+              printf ("The rounding integer (%ld) is%s representable in "
+                      "unsigned long,\nbut the erange flag is%s set.\n",
+                      s, Not, Not);
+              exit (1);
+            }
+        }
+    }
+
+  mpfr_clear (x);
+}
+
+/* Test mpfr_get_si and mpfr_get_ui, on values around some particular
+ * integers (see ts[] and tu[]): x = t?[i] + j/4, where '?' is 's' or
+ * 'u', and j is an integer from -8 to 8.
+ */
+static void get_tests (void)
+{
+  mpfr_exp_t emin, emax;
+  mpfr_t x, z;
+  long ts[5] = { LONG_MIN, LONG_MAX, -17, 0, 17 };
+  unsigned long tu[3] = { 0, ULONG_MAX, 17 };
+  int s, i, j, odd, ctr = 0;
+  int inex;
+  int r;
+
+  emin = mpfr_get_emin ();
+  emax = mpfr_get_emax ();
+
+  /* We need the bitsize of an unsigned long + 3 bits (1 additional bit for
+   * the cases >= ULONG_MAX + 1; 2 additional bits for the fractional part).
+   */
+  mpfr_init2 (x, sizeof (unsigned long) * CHAR_BIT + 3);
+
+  mpfr_init2 (z, MPFR_PREC_MIN);
+  mpfr_set_ui_2exp (z, 1, -2, MPFR_RNDN);  /* z = 1/4 */
+
+  for (s = 1; s >= 0; s--)
+    for (i = 0; i < (s ? 5 : 3); i++)
+      {
+        odd = (s ? (unsigned long) ts[i] : tu[i]) & 1;
+        inex = s ?
+          mpfr_set_si (x, ts[i], MPFR_RNDN) :
+          mpfr_set_ui (x, tu[i], MPFR_RNDN);
+        MPFR_ASSERTN (inex == 0);
+        inex = mpfr_sub_ui (x, x, 2, MPFR_RNDN);
+        MPFR_ASSERTN (inex == 0);
+        for (j = -8; j <= 8; j++)
+          {
+            /* Test x = t?[i] + j/4 in each non-RNDF rounding mode... */
+            RND_LOOP_NO_RNDF (r)
+              {
+                mpfr_flags_t ex_flags, flags;
+                int e, k, overflow;
+
+                ctr++;  /* for the check below */
+
+                /* Let's determine k such that the rounded integer should
+                   be t?[i] + k, assuming an unbounded exponent range. */
+                k = (j + 8 +
+                     (MPFR_IS_LIKE_RNDD (r, MPFR_SIGN (x)) ? 0 :
+                      MPFR_IS_LIKE_RNDU (r, MPFR_SIGN (x)) ? 3 :
+                      2)) / 4 - 2;
+                if (r == MPFR_RNDN && ((unsigned int) j & 3) == 2 &&
+                    ((odd + k) & 1))
+                  k--;  /* even rounding */
+
+                /* Overflow cases. Note that with the above choices:
+                   _ t?[0] == minval(type)
+                   _ t?[1] == maxval(type)
+                */
+                overflow = (i == 0 && k < 0) || (i == 1 && k > 0);
+
+                /* Expected flags. Note that in case of overflow, only the
+                   erange flag is set. Otherwise, the result is inexact iff
+                   j mod 1 != 0, i.e. the last two bits are not 00. */
+                ex_flags = overflow ? MPFR_FLAGS_ERANGE
+                  : ((unsigned int) j & 3) != 0 ? MPFR_FLAGS_INEXACT : 0;
+
+                mpfr_clear_flags ();
+
+#define GET_TESTS_TEST(TYPE,TZ,F,C,FMT)                                 \
+                do {                                                    \
+                  TYPE a, d;                                            \
+                                                                        \
+                  a = TZ[i] + (overflow ? 0 : k);                       \
+                  if (e)                                                \
+                    {                                                   \
+                      mpfr_exp_t ex;                                    \
+                      ex = MPFR_GET_EXP (x);                            \
+                      set_emin (ex);                                    \
+                      set_emax (ex);                                    \
+                    }                                                   \
+                  d = F (x, (mpfr_rnd_t) r);                            \
+                  flags = __gmpfr_flags;                                \
+                  set_emin (emin);                                      \
+                  set_emax (emax);                                      \
+                  if (flags != ex_flags || a != d)                      \
+                    {                                                   \
+                      printf ("Error in get_tests for " #F " on %s%s\n", \
+                              mpfr_print_rnd_mode ((mpfr_rnd_t) r),     \
+                              e ? ", reduced exponent range" : "");     \
+                      printf ("x = t" C "[%d] + (%d/4) = ", i, j);      \
+                      mpfr_out_str (stdout, 10, 0, x, MPFR_RNDN);       \
+                      printf ("\n--> k = %d\n", k);                     \
+                      printf ("Expected %l" FMT "\n", a);               \
+                      printf ("Got      %l" FMT "\n", d);               \
+                      printf ("Expected flags:");                       \
+                      flags_out (ex_flags);                             \
+                      printf ("Got flags:     ");                       \
+                      flags_out (flags);                                \
+                      exit (1);                                         \
+                    }                                                   \
+                } while (0)
+
+                for (e = 0; e < 2; e++)
+                  {
+                    if (e && MPFR_IS_ZERO (x))
+                      break;
+                    if (s)
+                      GET_TESTS_TEST (long,
+                                      ts, mpfr_get_si, "s", "d");
+                    else
+                      GET_TESTS_TEST (unsigned long,
+                                      tu, mpfr_get_ui, "u", "u");
+                  }
+              }
+            inex = mpfr_add (x, x, z, MPFR_RNDN);
+            MPFR_ASSERTN (inex == 0);
+          }
+      }
+
+  /* Check that we have tested everything: 8 = 5 + 3 integers t?[i]
+   * with 17 = 8 - (-8) + 1 additional terms (j/4) for each integer,
+   * and each non-RNDF rounding mode.
+   */
+  MPFR_ASSERTN (ctr == 8 * 17 * ((int) MPFR_RND_MAX - 1));
+
+  mpfr_clear (x);
+  mpfr_clear (z);
+}
+
 /* FIXME: Comparing against mpfr_get_si/ui is not ideal, it'd be better to
    have all tests examine the bits in mpfr_t for what should come out.  */
 
@@ -159,9 +343,11 @@ main (int argc, char *argv[])
 
   tests_start_mpfr ();
 
+  get_tests ();
+
   mpfr_init2 (x, 100);
 
-  N = (argc==1) ? 100000 : atol (argv[1]);
+  N = (argc == 1) ? 100000 : atol (argv[1]);
 
   for (k = 1; k <= N; k++)
     {
@@ -215,21 +401,17 @@ main (int argc, char *argv[])
 
   mpfr_set_prec (x, 3);
   inex = mpfr_set_si (x, 77617, MPFR_RNDD); /* should be 65536 */
-  if (MPFR_MANT(x)[0] != ((mp_limb_t)1 << (mp_bits_per_limb-1))
-      || inex >= 0)
+  if (MPFR_MANT(x)[0] != MPFR_LIMB_HIGHBIT || inex >= 0)
     {
       printf ("Error in mpfr_set_si(x:3, 77617, MPFR_RNDD)\n");
-      mpfr_print_binary (x);
-      puts ("");
+      mpfr_dump (x);
       exit (1);
     }
   inex = mpfr_set_ui (x, 77617, MPFR_RNDD); /* should be 65536 */
-  if (MPFR_MANT(x)[0] != ((mp_limb_t)1 << (mp_bits_per_limb-1))
-      || inex >= 0)
+  if (MPFR_MANT(x)[0] != MPFR_LIMB_HIGHBIT || inex >= 0)
     {
       printf ("Error in mpfr_set_ui(x:3, 77617, MPFR_RNDD)\n");
-      mpfr_print_binary (x);
-      puts ("");
+      mpfr_dump (x);
       exit (1);
     }
 
@@ -345,7 +527,7 @@ main (int argc, char *argv[])
   MPFR_ASSERTN (mpfr_get_ui (x, MPFR_RNDD) == 0);
   MPFR_ASSERTN (mpfr_get_si (x, MPFR_RNDD) == 0);
 
-  /* Test for ERANGE flag + correct behaviour if overflow */
+  /* Test for ERANGE flag + correct behavior if overflow */
   mpfr_set_prec (x, 256);
   mpfr_set_ui (x, ULONG_MAX, MPFR_RNDN);
   mpfr_clear_erangeflag ();
@@ -402,16 +584,16 @@ main (int argc, char *argv[])
     }
 
   mpfr_set_nan (x);
-  mpfr_clear_erangeflag ();
+  mpfr_clear_flags ();
   d = mpfr_get_ui (x, MPFR_RNDN);
-  if (d != 0 || !mpfr_erangeflag_p ())
+  if (d != 0 || __gmpfr_flags != MPFR_FLAGS_ERANGE)
     {
       printf ("ERROR for get_ui + NaN\n");
       exit (1);
     }
   mpfr_clear_erangeflag ();
   d = mpfr_get_si (x, MPFR_RNDN);
-  if (d != 0 || !mpfr_erangeflag_p ())
+  if (d != 0 || __gmpfr_flags != MPFR_FLAGS_ERANGE)
     {
       printf ("ERROR for get_si + NaN\n");
       exit (1);
@@ -459,6 +641,7 @@ main (int argc, char *argv[])
   test_2exp ();
   test_macros ();
   test_macros_keyword ();
+  test_get_ui_smallneg ();
   tests_end_mpfr ();
   return 0;
 }
